const char glsl_idx16_lut_bilinear_fsh_src[] =
"\n"
"#pragma optimize (on)\n"
"#pragma debug (off)\n"
"\n"
"uniform sampler2D color_texture;\n"
"uniform sampler2D colortable_texture;\n"
"uniform vec2      colortable_sz; // ct size\n"
"uniform vec2      colortable_pow2_sz; // pow2 ct size\n"
"uniform vec2      color_texture_pow2_sz; // pow2 tex size\n"
"\n"
"vec4 lutTex2D(in float x, in float y)\n"
"{\n"
"	vec2 texcoord = vec2(x, y);\n"
"	vec4 color_tex;\n"
"	vec2 color_map_coord;\n"
"\n"
"	// normalized texture coordinates ..\n"
"	color_tex = texture2D(color_texture, texcoord);\n"
"\n"
"	// GL_UNSIGNED_SHORT GL_ALPHA in ALPHA16 conversion:\n"
"	// general: f = c / ((2*N)-1), c color bitfield, N number of bits\n"
"	// ushort:  c = ((2**16)-1)*f;\n"
"	color_map_coord.x  = 65536.0 * color_tex.a;\n"
"\n"
"	// map it to the 2D lut table\n"
"	color_map_coord.y = floor(color_map_coord.x/colortable_sz.x);\n"
"	color_map_coord.x =   mod(color_map_coord.x,colortable_sz.x);\n"
"\n"
"	return texture2D(colortable_texture, color_map_coord/colortable_pow2_sz);\n"
"}\n"
"\n"
"vec4 f_bilinear(in vec2 xy)\n"
"{\n"
"	// mix(x,y,a): x*(1-a) + y*a\n"
"	//\n"
"	// bilinear filtering includes 2 mix:\n"
"	//\n"
"	//   pix1 = tex[x0][y0] * ( 1 - u_ratio ) + tex[x1][y0] * u_ratio\n"
"	//   pix2 = tex[x0][y1] * ( 1 - u_ratio ) + tex[x1][y1] * u_ratio\n"
"	//   fin  =    pix1     * ( 1 - v_ratio ) +     pix2    * v_ratio\n"
"	//\n"
"	// so we can use the build in mix function for these 2 computations ;-)\n"
"	//\n"
"	vec2 uv_ratio     = fract(xy*color_texture_pow2_sz); // xy*color_texture_pow2_sz - floor(xy*color_texture_pow2_sz);\n"
"	vec2 one          = 1.0/color_texture_pow2_sz;\n"
"\n"
"	return mix ( mix( lutTex2D(xy.x, xy.y      ), lutTex2D(xy.x+one.x, xy.y      ), uv_ratio.x),\n"
"	             mix( lutTex2D(xy.x, xy.y+one.y), lutTex2D(xy.x+one.x, xy.y+one.y), uv_ratio.x), uv_ratio.y );\n"
"}\n"
"\n"
"void main()\n"
"{\n"
"	gl_FragColor = f_bilinear(gl_TexCoord[0].st);\n"
"}\n"
"\n"
;
